package infomat.vectorspace;

import java.util.*;


/**
 * A IObjectGroup is a set of IObject:s.
 *
 * @param ordered A vector of the IObject:s.
 * @param objectOrderIndexMap A mapping from IObject to index in the vector "ordered".
 * @param objectOrderValueMap A mapping from IObject to a value. These values are the basis of the ordering in the vector "ordered".
 *
 * @version          070314
 * @author           Magnus Rosell
 */
public class IObjectGroup extends IObject {
	public static int maxIdSoFar=0;

	Vector<IObject> ordered; // The ordered vector
	public HashMap<IObject, Integer> objectOrderIndexMap; // The vector index
	HashMap<IObject, Float> objectOrderValueMap; // The basis for the ordering
	
	public IObjectGroup(String name){ 
		super(name, 1, ++maxIdSoFar); // 1 - IObjectGroup

		ordered = new Vector<IObject>();
		objectOrderIndexMap = new HashMap<IObject, Integer>();
		objectOrderValueMap = new HashMap<IObject, Float>();
	}

	public IObjectGroup(String name, int id){ 
		super(name, 1, id); // 1 - IObjectGroup
		if(id>maxIdSoFar)
			maxIdSoFar=id;

		ordered = new Vector<IObject>();
		objectOrderIndexMap = new HashMap<IObject, Integer>();
		objectOrderValueMap = new HashMap<IObject, Float>();
	}

	/**
	 * Used by IObjectGrouping and IObjectSet.
	 */
	protected IObjectGroup(String name, int kind, int id){ 
		super(name, kind, id); 
		ordered = new Vector<IObject>();
		objectOrderIndexMap = new HashMap<IObject, Integer>();
		objectOrderValueMap = new HashMap<IObject, Float>();
	}

	/**
	 * Copies references.
	 */
	/*
    public IObjectGroup(IObjectGroup cog){
	this.ordered = cog.ordered;
	this.objectOrderIndexMap=cog.objectOrderIndexMap;
	this.objectOrderValueMap=cog.objectOrderValueMap;
    }
	 */

	/**
	 * Adds a new IObject in the place the value implicates.
	 * For the object to get the right place the group has to be
	 * sorted through {@link #sortInOrderValueOrder()}.
	 */
	public void add(IObject co, float orderValue){
		objectOrderIndexMap.put(co, ordered.size());
		ordered.add(co);
		objectOrderValueMap.put(co, orderValue);
	}

	/**
	 * Adds a new IObject at the next place.
	 */
	public void add(IObject co){
		add(co, (float) ordered.size());
	}

	/**
	 * Adds all the CObjet:s at the end.
	 */
	protected void addAll(Vector<IObject> cobjects){
		for(int i=0; i<cobjects.size(); i++)
			add(cobjects.elementAt(i));
	}

	/**
	 * Removes teh IObject.
	 */
	protected void remove(IObject co){
		if(contains(co)){
			ordered.remove(co);
			objectOrderIndexMap.remove(co);
			objectOrderValueMap.remove(co);
		}
	}

	public void removeAll(){
		ordered = new Vector<IObject>();
		objectOrderIndexMap = new HashMap<IObject, Integer>();
		objectOrderValueMap = new HashMap<IObject, Float>();    	
	}

	/**
	 * Returns true when the IObjectGroup contains the IObject.
	 * False otherwise.
	 */
	public boolean contains(IObject co){
		return objectOrderIndexMap.containsKey(co);
	}

	/**
	 * Returns the number of IObjects in the IObjectGroup.
	 */
	public int numberOfIObjects(){
		return ordered.size();
	}

	/**
	 * Returns the name of thie IObjectGroup.
	 */
	public String getName(){
		return super.getString();
	}

	public void setName(String name){
		super.setString(name);
	}

	/**
	 * Returns the IObject at the i:the place according to the ordering.
	 * @see #sortInOrderValueOrder
	 */
	public IObject getIObjectAt(int i){
		return ordered.elementAt(i);
	}

	/**
	 * Returns the order number of the IObject.
	 * If it is not present -1 is returned.
	 * @see #sortInOrderValueOrder
	 */
	public int getIObjectOrderIndex(IObject co){
		/*
    	System.out.println("IObjectGroup - objectOrderIndexMap: "+
    			//model.getIMatrix().
    			objectOrderIndexMap);
		 */
		Integer i = objectOrderIndexMap.get(co);
		if(i==null)
			return -1;
		return i;
//		return objectOrderIndexMap.get(co);
	}

	/**
	 * Returns a vector of the ordered IObjects.
	 * @see #sortInOrderValueOrder
	 */
	public Vector<IObject> getOrdered(){
		return ordered;
	}

	/**
	 * Returns a HashMap with the objects mapped to the values that
	 * are the basis for the ordering.
	 * @return a HashMap
	 */
	public HashMap<IObject, Float> getObjectOrderValueMap(){
		return objectOrderValueMap;
	}
	
	/**
	 * Sorts the IObjects in the vector "ordered" 
	 * according to the values they have.
	 */
	public void sortInOrderValueOrder(){
		IObjectComparator coc = new IObjectComparator(this);
		Collections.sort(ordered, coc);

		objectOrderIndexMap = new HashMap<IObject, Integer>();
		for(int i=0; i<ordered.size(); i++){
			IObject co = ordered.elementAt(i);
			objectOrderIndexMap.put(co, i);
		}
	}

	/**
	 * Returns a centroid in the form of a HashMap, mapping
	 * IObject to a Float value.
	 */
	public HashMap<IObject, Float> getRepresentation(IMatrix matrix){
		HashMap<IObject, Float> centroid = new HashMap<IObject, Float>();
		for(int i=0; i<ordered.size(); i++){
			IObject co = ordered.elementAt(i);
			Vector features = matrix.getCellsWith(co);
			if(features!=null){
				for(int f=0; f<features.size(); f++){
					IMatrixCell sc = (IMatrixCell) features.elementAt(f);
					IObject feature = sc.getRowIObject();
					if(co==feature)
						feature = sc.getColumnIObject();

					Float w = centroid.get(feature);
					if(w!=null)
						centroid.put(feature, w+sc.getWeight());
					else
						centroid.put(feature, sc.getWeight());
				}
			}
		}

		Iterator<IObject> it = centroid.keySet().iterator();
		while(it.hasNext()){
			IObject key = it.next();
			centroid.put(key, centroid.get(key)/numberOfIObjects());
		}
		return centroid;
	}

	/**
	 * A private class that is used to order the IObjects.
	 * The order depends on the value in the "objectOrderValueMap"
	 * parameter of the IObjectGroup.
	 */
	private class IObjectComparator implements Comparator{

		private IObjectGroup cog;

		public IObjectComparator(IObjectGroup cog){
			this.cog=cog;
		}

		/**
		 * Compares two IObject:s and returns 1, 0 or -1 
		 *
		 * @param  o1 a IObject
		 * @param  o2 a IObject
		 * @return int 
		 */
		public int compare(Object o1, Object o2) {
			float value1 = cog.objectOrderValueMap.get((IObject) o1);
			float value2 = cog.objectOrderValueMap.get((IObject) o2);

			// Strst frst
			if(value1<value2)
				return 1;
			else if (value1>value2)
				return -1;
			else
				return 0;
		}    

	}



}

