package infomat.vectorspace;

import java.util.*;

/**
 * A IObjectSet is a complete set of IObject:s.
 * This can either be the complete set of objects one investigates
 * or the complete feature set by which these objects are represented.
 * In fact, this representations makes no difference between the two.
 *
 * @param groupings A vector of groupings of this IObjectSet.
 * @param nameGroupingMap A mapping of the names of the IObjectGroupings of this IObjectSet to the COBjectGroupings.
 * @param currentGrouping The current CobjectGrouping. This is the basis for the ordering of the IObject:s in theis COBjectSet right now.
 * @param idIObjectMap A mapping between IObjectId and IObject.
 *
 * @version          070314
 * @author           Magnus Rosell
 */
public class IObjectSet extends IObjectGroup
{
	private static int idIterator=0;

	private Vector<IObjectGrouping> groupings;
	private HashMap<String, IObjectGrouping> nameGroupingMap;
	private IObjectGrouping currentGrouping=null;
	private HashMap<Integer, IObject> idIObjectMap;

	public IObjectSet(String name){ 
		super(name, 3, idIterator++); // 3 - IObjectSet

		groupings = new Vector<IObjectGrouping>();
		nameGroupingMap = new HashMap<String, IObjectGrouping>();
		idIObjectMap = new HashMap<Integer, IObject>();
	}

	/**
	 * Adds a new IObject.
	 * OBS. This is not reflected in any IObjectGrouping:s 
	 * that already exits. 
	 */
	public void add(IObject co){
		super.add(co);
		idIObjectMap.put(co.getID(), co);
	}

	/** 
	 * Removes a IObject. It is removed from all IObjectGrouping:s as well.
	 * Compare to #add where IObjects are not added to existing 
	 * IObjectGrouping:s.
	 */
	protected void remove(IObject co){
		super.remove(co);
		idIObjectMap.remove(co.getID());

		if(groupings!=null){
			for(int i=groupings.size()-1; i>=0; i--){
				IObjectGrouping coging = groupings.elementAt(i);
				coging.remove(co);
				if(coging.numberOfIObjectGroups()<1) // Hnder detta r det fara  frde! :)
					removeGrouping(coging);
			}
		}

		/*
	SKa inte behvas - alla grupperingar ska finnas i vektorn.
	if(currentGrouping!=null)
	    currentGrouping.remove(co);
		 */
	}

	/**
	 * Removes and returns the IObjects in the range given,
	 * the range within the vector "ordered".
	 */
	/*
    protected Vector<IObject> remove(int from, int to){

	// Ta bort
	Vector<IObject> removed = new Vector<IObject>();
	for (int i=to;i>=from;i--){
	    IObject co = ordered.elementAt(i);
	    remove(co);
	    removed.add(co);
	}

	// Sortera om
	if(groupings!=null){
	    for(int i=0; i<groupings.size(); i++){
		IObjectGrouping coging = groupings.elementAt(i);
		coging.sortAllGroupsInOrderValueOrder();
	    }
	}
	return removed;
    }
	 */

	/**
	 * Adds an IObjectGrouping.
	 * Returns false if the name for this new grouping is already 
	 * taken. 
	 */
	public boolean addGrouping(IObjectGrouping cog){
		String name = cog.getName();
		if(nameGroupingMap.containsKey(name))
			return false;
		groupings.add(cog);
		nameGroupingMap.put(name, cog);
		return true;
	}

	/**
	 * Removes an IObjectGrouping.
	 */
	public void removeGrouping(IObjectGrouping cog){
		groupings.remove(cog);
		nameGroupingMap.remove(cog.getName());
	}

	/**
	 * Removes all IObjects that's not in the current IObjectGrouping
	 * from the set and all its IObjectGroupings.
	 */
	public void purgeSetToCurrentGrouping(){
		Vector<IObject> removed = new Vector<IObject>();
		for(int i=0; i<ordered.size(); i++){
			IObject io = ordered.elementAt(i);
			if(currentGrouping.getIObjectOrderIndex(io)==-1)
				removed.add(io);
		}
		for(int i=0; i<removed.size(); i++){
			IObject io = removed.elementAt(i);
			// remove from set
			remove(io); 
			// remove from groupings
			for(int j=0; j<groupings.size(); j++){
				groupings.elementAt(j).remove(io);
			}
		}
	}


	/**
	 * Sets the current IObjectGrouping.
	 * Returns false if the IObjectGrouping has a conflict
	 * as in #addGrouping(). 
	 */
	public boolean setGrouping(IObjectGrouping cog){
		boolean ret=true;
		if(cog!=null && !groupings.contains(cog))
			ret = addGrouping(cog);
		if(ret)
			currentGrouping=cog;
		return ret;
	}

	/**
	 * Returns the current IObjectGrouping.
	 */
	public IObjectGrouping getCurrentGrouping(){
		return currentGrouping;
	}

	/**
	 * Returns all IObjectGrouping:s of this IObjectSet.
	 */
	public Vector<IObjectGrouping> getGroupings(){
		return groupings;
	}

	/**
	 * Returns the IObjectGrouping with the name given.
	 */
	public IObjectGrouping getGroupingWithName(String name){
		return nameGroupingMap.get(name);
	}

	/**
	 * Sorts the vector "ordered" accorging to the IObjectGrouping given.
	 */
	/*
    public void sortAccordingToGrouping(IObjectGrouping coging){
	ordered = new Vector<IObject>();
	objectOrderIndexMap = new HashMap<IObject, Integer>();
	objectOrderValueMap = new HashMap<IObject, Float>();
	idIObjectMap = new HashMap<Integer, IObject>();
	Vector<IObjectGroup> groups = coging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    Vector<IObject> cobjects = groups.elementAt(i).getOrdered();
	    for(int j=0; j<cobjects.size(); j++){
		IObject co = cobjects.elementAt(j);
		add(co);
	    }
	}

    }
	 */

	/**
	 * Makes a IObjectGrouping consisting of one IObjectGroup and
	 * sets it to be the current IObjectGrouping. Used as a first 
	 * IObjectGrouping as many function needs one to work.
	 */
	public IObjectGrouping makeStartGrouping(){ 
		IObjectGrouping startGrouping = 
			new IObjectGrouping("Start Grouping for "+getString());
		int id=-1;
		IObjectGroup startGroup = 
			new IObjectGroup("Start Grouping Group", id); //, myIMatrix);
		startGrouping.add(startGroup);

		Vector<IObject> ordered = getOrdered();
		for(int i=0; i<ordered.size(); i++){
			IObject co = ordered.elementAt(i);
			startGroup.add(co);
		}

		startGrouping.sort();
		int i=1;
		while(!setGrouping(startGrouping)){
			startGrouping.setName(startGrouping.getName()+" ("+i+")");
			i++;
		}
		return startGrouping;
	}

	/**
	 * Returns the vector "ordered" in its current order.
	 * If it isn't assign it is ordered accoring to the current
	 * IObjectGrouping first.
	 */
	/*
    public Vector<IObject> getOrdered(){
	if(currentGrouping==null)
	    return ordered;
	else{
	    sortAccordingToGrouping(currentGrouping);
	    return ordered;
	}
    }
	 */

	/**
	 * Returns the IObject with the given id.
	 */
	protected IObject getIObjectWithId(int i){
		return idIObjectMap.get(i);
	}
}

