package mro.util;

import java.io.PrintWriter; 
import java.util.*; 
import java.lang.Thread;

/**
 * A UniqueIdentifier for an object based on the Class name and
 * two longs: the current system time and the current thread ID.
 *
 * @see Thread
 * @version          061130
 * @author           Magnus Rosell
 */
public class UniqueIdentifier implements XMLPartIO{

    /** A String containing the full name of the class of the owner object. */
    private String ownerClassName;
    /** The creation time of the UniqueIdentifier.*/
    private long time = System.currentTimeMillis();
    /** The Thread in which the UniqueIdentifier was created.*/
    private long threadID = ((Thread) Thread.currentThread()).getId();

    /**
     * Constructor.
     */
    public UniqueIdentifier(){}

    /**
     * Constructor.
     */
    public UniqueIdentifier(Object o){
	ownerClassName=o.getClass().getName();
    }

    /**
     * Copy constructor.
     */
    public UniqueIdentifier(UniqueIdentifier ui){
	this.ownerClassName=ui.ownerClassName;
	this.time=ui.time;
	this.threadID=ui.threadID;
    }

    /**
     * Constructor.
     */
    public UniqueIdentifier(long time, long threadID, Object o){
	this.time=time;
	this.threadID=threadID;
	ownerClassName=o.getClass().getName();
    }

    /**
     * Constructor.
     */
    public UniqueIdentifier(long time, long threadID, String ownerClassName){
	this.time=time;
	this.threadID=threadID;
	this.ownerClassName=ownerClassName;
    }

    /**
     * Returns the time for the creation this UniqueIdentifier.
     */
    public long getTime(){
	return time;
    }

    /**
     * Returns the ID of the Thread in which 
     * this UniqueIdentifier was created.
     */
    public long getThreadID(){
	return threadID;
    }

    /**
     * Returns the full name of the class of the owning object.
     */
    public String getOwnerClassName(){
	return ownerClassName;
    }

    /**
     * Returns a String with the name of the element defining a 
     * UniqueIdentifier in a DTD.
     */
    public String getDTDElementName(){
	return "UniqueIdentifier";
    }

    /**
     * Compares two UniqueIdentifiers.
     */
    public boolean equals(Object obj){
	UniqueIdentifier ui = (UniqueIdentifier) obj;
	if(ownerClassName.equals(ui.getOwnerClassName()) &&
	   time==ui.getTime() &&
	   threadID==ui.getThreadID())
	    return true;
	return false;
    }

    /**
     * Just for printing to screen. Debugging essentially.
     */
    public String toString(){
	String ret = "UniqueIdentifier\n";
	ret+="  ownerClassName: "+ownerClassName+"\n";
	ret+="  time: "+time+"\n";
	ret+="  threadID: "+threadID+"\n";
	return ret;
    }

    /*************** XML ******************/

    /**
     * Prints the DTD for a UniqueIndentifier. Use in the context of a larger
     * DTD. The DTD looks like this:
     * <pre>
     * &lt;!--A Unique Identifier--&gt;
     * &lt;!ELEMENT UniqueIdentifier (OwnerClassName, Time, Thread)&gt;
     * &lt;!ELEMENT OwnerClassName (#PCDATA)&gt; &lt;!--Class of the object it identifies--&gt;
     * &lt;!ELEMENT Time (#PCDATA)&gt; &lt;!--System Time of Creation--&gt;
     * &lt;!ELEMENT Thread (#PCDATA)&gt; &lt;!--ThreadID--&gt;
     * </pre>
     */
    public static void staticPrintPartDTD(PrintWriter out){
 	out.println("<!--A Unique Identifier-->");
	out.println("<!ELEMENT UniqueIdentifier (OwnerClassName, Time, Thread)>");
        out.println("<!ELEMENT OwnerClassName (#PCDATA)> <!--Class of the object it identifies-->");
        out.println("<!ELEMENT Time (#PCDATA)> <!--System Time of Creation-->");
        out.println("<!ELEMENT Thread (#PCDATA)> <!--ThreadID-->");
    }

    public void printPartDTD(PrintWriter out){
	UniqueIdentifier.staticPrintPartDTD(out);
    }

    /**
     * Prints this UniqueIdentifier to a PrintWriter as a part of a larger
     * XML-file following the DTD in {@link #printPartDTD}.
     */
    public void printPartXML(PrintWriter out){
        out.println("<UniqueIdentifier>");
	out.println("<OwnerClassName>"+ownerClassName+"</OwnerClassName>");
	out.println("<Time>" + time + "</Time>");
	out.println("<Thread>" + threadID + "</Thread>");
        out.println("</UniqueIdentifier>");
    }

    /**
     * Use when defining a {@link org.xml.sax.helpers.DefaultHandler}
     * to parse a larger XML-file where a UniqueIdentifier is a part.
     * This particular instance always returns false, as no
     * information on the start of these tags are nedeed.
     */
    public boolean startElementPartXML(StringBuffer buffer, 
				       String qualifiedName){
	return false;
    }
        
    /**
     * Use when defining a {@link org.xml.sax.helpers.DefaultHandler}
     * to parse a larger XML-file where a UniqueIdentifier is a part.
     * Returns true when the elements in the DTD 
     * (see {@link #printPartDTD}) are found.
     * In these cases the content of the UniqueIdentifier are
     * set to the the values found in the XML-file.
     */
    public boolean endElementPartXML(StringBuffer buffer, 
				     String qualifiedName){
	if(qualifiedName.equals("Time")){
	    time = Long.parseLong(buffer.toString());
	    return true;
	}
	else if(qualifiedName.equals("Thread")){
	    threadID = Long.parseLong(buffer.toString());
	    return true;
	}
	else if(qualifiedName.equals("OwnerClassName")){
	    ownerClassName = buffer.toString();
	    return true;
	}
	return false;
   }
}
